defmodule Yahtzee do
 def score_upper(dice) do
    1..6
    |> Enum.map(&score_face(dice, &1))
    |> Enum.into(%{})
  end

  defp score_face(dice, face) do
    key = face_to_atom(face)
    value = length(Enum.filter(dice, fn e -> e == face end))
    {key, value}
  end

  defp face_to_atom(face) do
    case face do
      1 -> :Ones
      2 -> :Twos
      3 -> :Threes
      4 -> :Fours
      5 -> :Fives
      6 -> :Sixes
    end
  end


  def score_lower(dice) do
    yahtzee(dice) ||
    four_of_a_kind(dice) ||
    full_house(dice) ||
    three_of_a_kind(dice) ||
    large_straight(dice) ||
    small_straight(dice) ||
    chance(dice) ||
    %{}
end

  # Part 2
  defp three_of_a_kind(dice) do
    dice
    |> Enum.group_by(fn x -> x end)
    |> Enum.map(fn {face, occurrences} -> {face, length(occurrences)} end)
    |> Enum.filter(fn {_face, count} -> count >= 3 end)
    |> case do
      [{_face, _count}] -> %{"Three of a kind" => Enum.sum(dice)}
      _ -> nil
    end
  end

  # Part 3
  defp four_of_a_kind(dice) do
    grouped = Enum.frequencies(dice)
    if Enum.any?(grouped, fn {_face, count} -> count >= 4 end) do
      %{"Four of a kind" => Enum.sum(dice)}
    else
      nil
    end
  end

  defp full_house(dice) do
    grouped = Enum.frequencies(dice)
    if Enum.count(grouped, fn {_face, count} -> count == 3 end) == 1 and
       Enum.count(grouped, fn {_face, count} -> count == 2 end) == 1 do
      %{"Full house" => 25}
    else
      nil
    end
  end

  #Part 4, starts here
  defp large_straight?(dice) do
    [1, 2, 3, 4, 5] -- dice == [] or [2, 3, 4, 5, 6] -- dice == []
  end

  defp small_straight?(dice) do
    Enum.any?([[1, 2, 3, 4], [2, 3, 4, 5], [3, 4, 5, 6]], fn seq ->
      Enum.all?(seq, fn num -> Enum.member?(dice, num) end)
    end)
  end

  defp large_straight(dice) do
    if large_straight?(dice) do
      %{"Large straight" => 40}
    else
      nil
    end
  end

  defp small_straight(dice) do
    if small_straight?(dice) do
      %{"Small straight" => 30}
    else
      nil
    end
  end

  #Part 5, starts here
  defp yahtzee(dice) do
    unique_faces = Enum.uniq(dice)
    if length(unique_faces) == 1 do
      %{Yahtzee: 50}
    else
      nil
    end
  end

  defp chance(dice) do
    case score_lower_without_chance(dice) do
      %{} -> %{Chance: Enum.sum(dice)}
      _ -> nil
    end
  end

  defp score_lower_without_chance(dice) do
      yahtzee(dice) ||
      four_of_a_kind(dice) ||
      full_house(dice) ||
      three_of_a_kind(dice) ||
      large_straight(dice) ||
      small_straight(dice) ||
      %{}
  end

  #Part 6, start here


end
