defmodule YahtzeeLowerSectionTest do
  use ExUnit.Case

  def generate(dice_face, occurrences) do
    Enum.to_list(1..6)
    |> List.delete(dice_face)
    |> Enum.shuffle
    |> Enum.take(5 - occurrences)
    |> Enum.concat(List.duplicate(dice_face, occurrences))
    |> Enum.shuffle
  end

  test "Identify 'Three of a kind' with ones" do
    dices = generate(1, 3)
    sum = Enum.sum(dices)
    assert %{"Three of a kind" => ^sum} = Yahtzee.score_lower(dices)
  end

  test "Identify 'Three of a kind' with all the others" do
    Enum.map(2..6, fn (dice_face) ->
      dices = generate(dice_face, 3)
      sum = Enum.sum(dices)
      assert %{"Three of a kind" => ^sum} = Yahtzee.score_lower(dices)
    end)
  end

  #Part 3 of assignment, starts here
  test "Identify 'Full house' with every face" do
    Enum.map(1..6, fn _ ->
      [x,y] =
        Enum.shuffle(1..6)
        |> Enum.take(2)
      assert %{"Full house" => 25} = Yahtzee.score_lower([x, x, x, y, y] |> Enum.shuffle())
    end)
  end

  #Part 3 Full House
  test "Identify 'Four of a kind' with every face" do
    Enum.map(1..6, fn (dice_face) ->
      dices = generate(dice_face, 4)
      sum = Enum.sum(dices)
      assert %{"Four of a kind" => ^sum} = Yahtzee.score_lower(dices)
    end)
  end

  #Part 4, starts here
  test "Identify 'Large straight'" do
    combinations = [
      [1, 2, 3, 4, 5],
      [2, 3, 4, 5, 6]
    ]

    Enum.each(combinations, fn dices ->
      shuffled_dices = Enum.shuffle(dices)
      assert %{"Large straight" => 40} = Yahtzee.score_lower(shuffled_dices)
    end)
  end

  test "Identify 'Small straight' but no 'Large straight'" do
    combinations = [
      [2, 3, 2, 5, 4],
      [1, 3, 4, 5, 6]
    ]

    Enum.each(combinations, fn dices ->
      shuffled_dices = Enum.shuffle(dices)
      assert %{"Small straight" => 30} = Yahtzee.score_lower(shuffled_dices)
    end)
  end

  #Part 5
  test "Identify 'Yahtzee'" do
    Enum.map(1..6, fn n ->
      assert %{Yahtzee: 50} = Yahtzee.score_lower(List.duplicate(n,5))
    end)
  end

  test "Identify any other combination" do
    Enum.map(1..6, fn _ ->
      [x,y,z] =
        Enum.shuffle(1..6)
        |> Enum.take(3)
      seq = Enum.shuffle([x,x,y,y,z])
      sum = Enum.sum(seq)
      assert %{Chance: ^sum} = Yahtzee.score_lower(seq)
    end)
  end

  #Part 6
# Test Case 1: For dice configuration [2][2][5][5][5]
test "Score categories for dice configuration [2][2][5][5][5]" do
  expected_scores = %{
    "Twos" => 4,
    "Fives" => 15,
    "Three of a kind" => 19,
    "Full house" => 25,
    "Chance" => 19,
    "Yahtzee" => 0,
    "Four of a kind" => 0,
    "Large straight" => 0,
    "Small straight" => 0
  }

  assert expected_scores == Yahtzee.score_all_categories([2, 2, 5, 5, 5])
end

# Test Case 2: For dice configuration [4][4][4][4][5]
test "Score categories for dice configuration [4][4][4][4][5]" do
  expected_scores = %{
    "Fours" => 16,
    "Fives" => 5,
    "Three of a kind" => 21,
    "Four of a kind" => 21,
    "Chance" => 21,
    "Yahtzee" => 0,
    "Full house" => 0,
    "Large straight" => 0,
    "Small straight" => 0
  }

  assert expected_scores == Yahtzee.score_all_categories([4, 4, 4, 4, 5])
end

# Test Case 3: For dice configuration [1][2][3][4][5]
test "Score categories for dice configuration [1][2][3][4][5]" do
  expected_scores = %{
    "Ones" => 1,
    "Twos" => 2,
    "Threes" => 3,
    "Fours" => 4,
    "Fives" => 5,
    "Small straight" => 30,
    "Chance" => 15,
    "Yahtzee" => 0,
    "Three of a kind" => 0,
    "Four of a kind" => 0,
    "Full house" => 0,
    "Large straight" => 0
  }

  assert expected_scores == Yahtzee.score_all_categories([1, 2, 3, 4, 5])
end


end
